/*
 * Clases para manejar hitboxes en SDL2
 */
#ifndef _SDL_HITBOX
#define _SDL_HITBOX

#include <SDL.h>
#include "sdl_rect.h"
#include "sdl_circle.h"

namespace sdl {

enum shapeType {
    rectangle=0,
    circle
};

class Shape {
protected:
    shapeType type;
    union {
        FRect re;
        FCircle ci;
    };
public:
    Shape() : type(rectangle), re({0,0,0,0}) {}
    Shape(FCircle c) : type(circle), ci(c) {}
    Shape(FRect r) : type(rectangle), re(r) {}
    Shape Move(float alfa, sdl::FPoint &displacement, sdl::FPoint &escale);
    FCircle &Ci() { return ci; }
    FRect &Re() { return re; }
    friend class Hitbox;
};

Shape Shape::Move(float alfa, sdl::FPoint &displacement, sdl::FPoint &escale) {
    FPoint point;

    if(type == circle) {
        point = ci.Center().Move(alfa, displacement, escale);
        return Shape(FCircle(point, ci.R()*escale.X()));
    }
    //rectangulo:
    point = re.Corner().Move(alfa, displacement, escale);
    return Shape(FRect(point.X(), point.Y(), re.W()*escale.X(), re.H()*escale.Y()));
}

class Hitbox {
protected:
    std::vector<Shape> hitbox; // formas de hitbox normalizadas
    std::vector<Shape> hitboxT; // hitbox trasladadas
    bool CollisionCircleCircle(FCircle &c1, FCircle &c2);
    bool CollisionRectRect(FRect &r1, FRect &r2);
    bool CollisionRectCircle(FRect &r1, FCircle &ci);

public:
    Hitbox() {}
    void ResizeT() { hitboxT.resize(hitbox.size()); }
    void InsertShape(const Shape &f) { hitbox.push_back(f); }
    bool Collision(Hitbox &);
    void Move(float alfa, sdl::FPoint &displacement, sdl::FPoint &escale);
    void Reset() {
        hitbox.clear();
    }
    Shape GetShape(int i) { return hitboxT[i]; }
};

bool Hitbox::CollisionCircleCircle(FCircle &c1, FCircle &c2) {
    if(((c1.R()+c2.R())*(c1.R()+c2.R())) >=
       ((c1.Center().X()-c2.Center().X())*(c1.Center().X()-c2.Center().X())+
        (c1.Center().Y()-c2.Center().Y())*(c1.Center().Y()-c2.Center().Y())))
        return true;
    return false;
}

bool Hitbox::CollisionRectRect(FRect &r1, FRect &r2) {
    return SDL_HasIntersectionF(r1.get(), r2.get());
}

bool Hitbox::CollisionRectCircle(FRect &re, FCircle &ci) {
    FPoint pt = ci.Center();
    if(pt.X() < re.X()) pt.X() = re.X();
    if(pt.Y() < re.Y()) pt.Y() = re.Y();
    if(pt.X() > re.X()+re.W()) pt.X() = re.X()+re.W();
    if(pt.Y() > re.Y()+re.H()) pt.Y() = re.Y()+re.H();
    return ((pt.X()-ci.X())*(pt.X()-ci.X())+(pt.Y()-ci.Y())*(pt.Y()-ci.Y())) <= ci.R()*ci.R();
}

bool Hitbox::Collision(Hitbox &hitbox2) {
    for(size_t i=0; i<hitboxT.size(); i++) {
        for(size_t j=0; j<hitbox2.hitboxT.size(); j++) {
            if(hitbox[i].type==circle && hitbox2.hitbox[j].type==circle) {
                return CollisionCircleCircle(hitboxT[i].ci, hitbox2.hitboxT[j].ci);
            } else
            if(hitbox[i].type==rectangle && hitbox2.hitbox[j].type==rectangle) {
                return CollisionRectRect(hitboxT[i].re, hitbox2.hitboxT[j].re);
            } else
            if(hitbox[i].type==rectangle) {
                return CollisionRectCircle(hitboxT[i].re, hitbox2.hitboxT[j].ci);
            } else
                return CollisionRectCircle(hitbox2.hitboxT[j].re, hitboxT[i].ci);
        }
    }
    return false;
}

void Hitbox::Move(float alfa, sdl::FPoint &displacement, sdl::FPoint &escale) {
    for(size_t i = 0; i < hitbox.size(); i++) {
        hitboxT[i] = hitbox[i].Move(alfa, displacement, escale);
    }
}

} // namespace sdl

#endif // _SDL_HITBOX
